import Qt 4.7
import "scripts/mainscripts.js" as Scripts
import "scripts/settings.js" as Settings

Rectangle {
    id: window

    Component.onCompleted: Scripts.restoreSettings();

    property string _UPLOADS_FEED : "http://gdata.youtube.com/feeds/api/users/default/uploads?v=2&max-results=50"
    property string _FAVOURITES_FEED : "http://gdata.youtube.com/feeds/api/users/default/favorites?v=2&max-results=50"
    property string _PLAYLISTS_FEED : "http://gdata.youtube.com/feeds/api/users/default/playlists?v=2&max-results=50"
    property string _SUBSCRIPTIONS_FEED : "http://gdata.youtube.com/feeds/api/users/default/subscriptions?v=2&max-results=50"
    property string _NEW_SUB_VIDEOS_FEED : "http://gdata.youtube.com/feeds/api/users/default/newsubscriptionvideos?v=2&max-results=50"
    property string _MOST_RECENT_FEED : "http://gdata.youtube.com/feeds/api/standardfeeds/most_recent?v=2&max-results=50"
    property string _MOST_VIEWED_FEED : "http://gdata.youtube.com/feeds/api/standardfeeds/most_viewed?v=2&max-results=50&time=today"

    /* Theme variables */

    property string cuteTubeTheme : "dark"
    property string _ACTIVE_COLOR_HIGH : "#6382c6"
    property string _ACTIVE_COLOR_LOW : "#3d6be0"
    property string _BACKGROUND_COLOR : (cuteTubeTheme == "light") ? "#faf9f6" : "black"
    property string _GRADIENT_COLOR_HIGH : (cuteTubeTheme == "light") ? "#faf9f6" : "#524e4e"
    property string _GRADIENT_COLOR_LOW : (cuteTubeTheme == "light") ? "#edece8" : "black"
    property string _TEXT_COLOR : (cuteTubeTheme == "light") ? "black" : "white"
    property int _SMALL_FONT_SIZE : 18
    property int _STANDARD_FONT_SIZE : 24
    property int _LARGE_FONT_SIZE : 36



    function userIsSignedIn() {
        /* Check if the user is signed in */

        var signedIn = false;
        if (YouTube.currentUser != "") {
            signedIn = true;
        }
        else if (Settings.getDefaultAccount() != "unknown") {
            Scripts.signInToDefaultAccount();
            signedIn = true;
        }
        return signedIn;
    }

    function displayMessage(message) {
        /* Display a notification using the message banner */

        messageBanner.source = "";
        messageBanner.source = "MessageBanner.qml";
        messageBanner.item.message = message;
        toggleBusy(false);
    }

    function toggleBusy(isBusy) {
        notificationArea.isBusy = isBusy;
    }

    function toggleControls(showControls) {
        /* Show/hide the notification area and menu bar */

        if (showControls) {
            controlsTimer.running = true;
        }
        else {
            notificationArea.visible = false;
            menu.visible = false
        }
    }

    function createDownloadItem(video, convertToAudio) {
        /* Check for duplicate filepaths and create a new download item */

        var downloadItem = null;
        var path = Settings.getSetting("downloadPath") + video.title.replace(/[\"@&~=\/:?#!|<>*^]/g, "_") + ".mp4";
        if (convertToAudio) {
            path = path.slice(0, -4) + "(Audio)" + ".mp4";
        }
        var duplicate = false;
        var i = 0;
        while ((i < downloadModel.count) && (!duplicate)) {
            duplicate = (downloadModel.get(i).filePath == path);
            i++;
        }
        if (!duplicate) {
            var downloadItem = {
                filePath: path,
                title: video.title,
                thumbnail: video.thumbnail,
                playerUrl: video.playerUrl.split("&")[0],
                status: Settings.getSetting("downloadStatus"),
                quality: "",
                isNew: 1,
                convert: convertToAudio,
                bytesReceived: 0,
                totalBytes: 100,
                speed: ""
        }
        }
        return downloadItem;
    }

    function addDownload(video) {
        /* Add a video to the download list model */

        var downloadItem = createDownloadItem(video, false);
        if (downloadItem) {
            Settings.storeDownload(downloadItem);
            downloadModel.appendDownload(downloadItem);
            displayMessage(qsTr("Video(s) added to download queue"));
        }
    }

    function addAudioDownload(video) {
        /* Add a video to the download list model and set 'convert' to true */

        var downloadItem = createDownloadItem(video, true);
        if (downloadItem) {
            Settings.storeDownload(downloadItem);
            downloadModel.appendDownload(downloadItem);
            displayMessage(qsTr("Audio track(s) added to download queue"));
        }
    }

    onStateChanged: {
        if (messageBanner.source != "") {
            messageBanner.item.state = window.state;
        }
    }

    width: 800
    height: 480
    anchors.fill: parent
    color: _BACKGROUND_COLOR

    Timer {
        id: controlsTimer

        interval: 500
        onTriggered: {
            notificationArea.visible = true;
            menu.visible = true;
        }
    }

    Connections {
        /* Connect to signals from C++ object YouTube */

        target: YouTube

        onAlert: displayMessage(message)
        onAddedToFavourites: displayMessage(qsTr("Video added to favourites"))
        onAddedToPlaylist: displayMessage(qsTr("Video added to playlist"))
        onGotVideoUrl: Controller.playVideo(videoUrl)
        onAccessTokenChanged: {
            Scripts.getSubscriptions();
            Scripts.getPlaylists();
            Settings.setSetting("ytAccessToken", token);
        }
        onPlaylistCreated: {
            displayMessage(qsTr("New playlist created"));
            Scripts.getPlaylists();
        }
        onSubscribed: Scripts.getSubscriptions()
        onUnsubscribed: Scripts.getSubscriptions()
    }

    Connections {
        /* Connect to signals from C++ object DownloadManager */

        target: DownloadManager
        onAlert: displayMessage(message)
        onDownloadCompleted: {
            var downloadItem = downloadModel.get(downloadModel.currentDownload);
            Settings.removeStoredDownload(downloadItem.filePath);
            downloadModel.setProperty(downloadModel.currentDownload, "filePath", filename);
            if (downloadItem.convert) {
                Controller.convertToAudio(filename);
            }
            else {
                Scripts.moveToArchive(downloadItem);
            }
        }
        onDownloadCancelled: {
            downloadModel.deleteDownload(downloadModel.currentDownload);
            downloadModel.getNextDownload();
        }
        onQualityChanged: downloadModel.setProperty(downloadModel.currentDownload, "quality", downloadModel.qualityDict[quality])
        onStatusChanged: downloadModel.setProperty(downloadModel.currentDownload, "status", status)
        onProgressChanged: {
            downloadModel.setProperty(downloadModel.currentDownload, "bytesReceived", bytesReceived);
            downloadModel.setProperty(downloadModel.currentDownload, "totalBytes", bytesTotal);
            downloadModel.setProperty(downloadModel.currentDownload, "speed", speed);
        }
    }

    Connections {
        target: Controller

        onAlert: displayMessage(message)
        onConversionStarted: downloadModel.setProperty(downloadModel.currentDownload, "status", "converting");
        onConversionCompleted: {
            Settings.removeStoredDownload(downloadModel.get(downloadModel.currentDownload).filePath);
            downloadModel.setProperty(downloadModel.currentDownload, "filePath", downloadModel.get(downloadModel.currentDownload).filePath.replace(".mp4", ".m4a"));
            downloadModel.setProperty(downloadModel.currentDownload, "quality", qsTr("Audio"));
            Scripts.moveToArchive(downloadModel.get(downloadModel.currentDownload));
        }
        onConversionFailed: {
            var downloadItem = downloadModel.get(downloadModel.currentDownload);
            displayMessage(qsTr("Conversion of '") + downloadItem.title + qsTr("' failed"));
            Settings.removeStoredDownload(downloadItem.filePath);
            Scripts.moveToArchive(downloadItem);
        }
    }

    ListModel {
        /* Download list model */
        id: downloadModel

        property int currentDownload
        property variant qualityDict
        property variant statusDict

        Component.onCompleted: {
            downloadModel.qualityDict = { "hq": qsTr("High quality"), "mobile": qsTr("Mobile"), "360p": qsTr("360p"), "480p": qsTr("480p"), "720p": qsTr("720p") };
            downloadModel.statusDict = { "paused": qsTr("Paused"), "queued": qsTr("Queued"), "downloading": qsTr("Downloading"), "failed": qsTr("Failed") };
        }

        function appendDownload(downloadItem) {
            downloadModel.insert(0, downloadItem);
            if ((downloadItem.status == "queued") && (!DownloadManager.isDownloading)) {
                DownloadManager.startDownload(downloadItem.filePath, downloadItem.playerUrl);
                downloadModel.currentDownload = 0;
            }
        }

        function resumeDownload(index) {
            downloadModel.setProperty(index, "status", "queued");
        }

        function pauseDownload(index) {
            if ((index == downloadModel.currentDownload) && (DownloadManager.isDownloading)) {
                DownloadManager.pauseDownload();
            }
            else {
                downloadModel.setProperty(index, "status", "paused");
            }
        }

        function cancelDownload(index) {
            if ((index == downloadModel.currentDownload) && (DownloadManager.isDownloading)) {
                DownloadManager.cancelDownload();
            }
            else {
                deleteDownload(index);
            }
        }

        function deleteDownload(index) {
            var itemToDelete = downloadModel.get(index);
            var downloadItem;
            var deleted = false;
            var i = 0;
            while ((i < downloadModel.count) && (!deleted)) {
                downloadItem = downloadModel.get(i);
                if (downloadItem == itemToDelete) {
                    Settings.removeStoredDownload(downloadItem.filePath);
                    downloadModel.remove(i);
                }
                i++;
            }
        }

        function getNextDownload() {
            var i = downloadModel.count - 1;
            while ((i >= 0) && (!DownloadManager.isDownloading)) {
                var downloadItem = downloadModel.get(i);
                if (downloadItem.status == "queued") {
                    DownloadManager.startDownload(downloadItem.filePath, downloadItem.playerUrl);
                    downloadModel.currentDownload = i;
                }
                i--;
            }
        }
    }

    ListModel {
        /* Archive list model */

        id: archiveModel
    }

    PlaylistModel {
        /* Holds the users YouTube playlists */

        id: playlistModel

        onStatusChanged: {
            if ((playlistModel.status == XmlListModel.Ready) &&
                    (playlistModel.totalResults > 50) &&
                    (playlistModel.totalResults > playlistModel.count)) {
                Scripts.appendPlaylists();
            }
        }
    }

    SubscriptionsModel {
        /* Holds the users YouTube subscriptions */

        id: subscriptionsModel

        onStatusChanged: {
            if ((subscriptionsModel.status == XmlListModel.Ready) &&
                    (subscriptionsModel.totalResults > 50) &&
                    (subscriptionsModel.totalResults > subscriptionsModel.count)) {
                Scripts.appendSubscriptions();
            }
        }
    }

    NotificationArea {
        id: notificationArea

        property variant titleList : ["cuteTube"]

        function addTitle(title) {
            var titles = titleList;
            titles.push(title);
            titleList = titles;
        }

        function removeTitle() {
            var titles = titleList;
            titles.pop();
            titleList = titles;
        }

        z: 100
        viewTitle: titleList[windowView.currentIndex]

        Connections {
            onStartSearch: Scripts.search(query, order)
            onGoToVideo: Scripts.loadVideoInfo(video)
            onGoToDownloads: {
                if (windowView.currentIndex != viewsModel.count - 1) {
                    Scripts.loadDownloads()
                }
            }
        }
    }

    Loader {
        id: messageBanner

        width: window.width
        anchors.bottom: window.top
        z: 1
        onLoaded: {
            messageBanner.item.state = window.state;
            timer.running = true;
            messageBanner.state = "show"
        }

        Timer {
            id: timer

            interval: 2500
            onTriggered: messageBanner.state = ""
        }

        states: State {
            name: "show"
            AnchorChanges { target: messageBanner; anchors { bottom: undefined; top: notificationArea.bottom } }
        }

        transitions: Transition {
            AnchorAnimation { easing.type: Easing.OutQuart; duration: 500 }
        }

    }

    VisualItemModel {
        id: viewsModel

        HomeView {
            id: homeView

            width: window.width
            height: window.height

            Connections {
                onMyChannel: Scripts.loadAccountView()
                onMostRecent: Scripts.loadVideos(_MOST_RECENT_FEED, qsTr("Most Recent"))
                onMostViewed: Scripts.loadVideos(_MOST_VIEWED_FEED, qsTr("Most Viewed"))
                onArchive: Scripts.loadArchiveView()
            }
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }

        Loader {
            width: window.width
            height: window.height
        }
    }

    ListView {
        id: windowView

        anchors.fill: window
        model: viewsModel
        orientation: ListView.Horizontal
        interactive: false
        highlightMoveDuration: 300
        onCurrentIndexChanged: windowView.currentItem.visible = true
        focus: true

        MenuBar {
            id: menu

            z: 1
            onBackClicked: Scripts.goToPreviousView()
            onHomeClicked: Scripts.goHome()
            onQuitClicked: DownloadManager.isDownloading ? homeView.showConfirmExitDialog() : Qt.quit()
        }
    }

    states: State {
        name: "portrait"
        when: window.height > window.width
    }
}
